void Main(){
    startnew(WatchReviewQueueCoro);
    startnew(UpdateReviewServerActive);
}

const string PluginIcon = "\\$ff3" + Icons::StickyNoteO + "\\$z";
const string PluginName = Meta::ExecutingPlugin().Name;
const string MenuTitle = PluginIcon + "\\$s Map Review";

/** Render function called every frame intended only for menu items in the main menu of the `UI`.
*/
void RenderMenuMain() {
    if (UI::BeginMenu(MenuTitle + GetMenuStatus() + "###menu-"+PluginName)) {
        if (UI::MenuItem("Quick Join Review Server")) {
            startnew(QuickJoinReviewServer);
        }
        UI::Text("Waiting Time: " + CurrSubmitWaitingStr(QTF::Minutes_To_Wait));
        UI::Text("Maps in Queue: " + CurrSubmitWaitingStr(QTF::Maps_In_Queue));
        UI::Separator();
        UI::Text("Players Last Updated: " + Time::Format(Time::Now - g_LastReviewServerUpdate, false));
        UI::BeginDisabled(isUpdateReviewServerRunning);
        if (UI::MenuItem("Refresh Review Server (Works on main menu)")) {
            startnew(UpdateReviewServerActive);
        }
        UI::EndDisabled();
        UI::Separator();
        UI::Text("Players: " + serverPlayers.Length);
        UI::Indent();
        for (uint i = 0; i < serverPlayers.Length; i++) {
            UI::Text(serverPlayers[i]);

        }
        UI::Unindent();
        UI::Separator();
        UI::Text("Maps:");
        UI::Indent();
        for (uint i = 0; i < serverMaps.Length; i++) {
            UI::Text(serverMaps[i]);
        }
        UI::Unindent();
        UI::Separator();

        UI::EndMenu();
    }
}

string GetMenuStatus() {
    if (S_ShowQueueTimeInMenu) {
        return "\\$z\\$888 ("+CurrSubmitWaitingStr(S_MenuQueueTimeFormat)+")";
    }
    return "";
}

string CurrSubmitWaitingStr(QTF fmt) {
    if (lastReviewWaitTime < 0) {
        return '??';
    }
    auto sinceUpdate = (int(Time::Now) - lastReviewQCheck) / 1000;
    float minutesLeft = float(lastReviewWaitTime - 1 - sinceUpdate) / 60.;
    if (fmt == QTF::Minutes_To_Wait) {
        return tostring(int(Math::Ceil(minutesLeft))) + " min";
    } else if (fmt == QTF::Maps_In_Queue) {
        return tostring(int(Math::Ceil(minutesLeft / 3.))) + " map" + ((0.5 > minutesLeft || minutesLeft > 3.) ? "s" : "");
    }
    return "Unk Format: " + tostring(fmt);
}


uint lastReviewQCheck = 0;
int lastReviewWaitTime = -1;

void WatchReviewQueueCoro() {
    while (true) {
        yield();
        while (!S_ShowQueueTimeInMenu) yield();
        lastReviewWaitTime = Live::GetReviewServerWaitTime(S_DefaultReviewType);
        lastReviewQCheck = Time::Now;
        while (S_ShowQueueTimeInMenu && Time::Now < lastReviewQCheck + 60000) yield();
    }
}

void WatchReviewServerPopulation() {

}

uint g_LastReviewServerUpdate = 0;
string[] serverPlayers;
string[] serverMaps;
string[] serverUids;
bool isUpdateReviewServerRunning = false;

void UpdateReviewServerActive() {
    if (isUpdateReviewServerRunning) return;
    isUpdateReviewServerRunning = true;
    g_LastReviewServerUpdate = Time::Now;
    serverPlayers.RemoveRange(0, serverPlayers.Length);
    serverMaps.RemoveRange(0, serverMaps.Length);
    serverUids.RemoveRange(0, serverUids.Length);
    auto title = cast<CGameManiaPlanet>(GetApp()).ManiaTitleControlScriptAPI;
    while (!title.IsReady) yield();
    auto x = Live::GetReviewServerConnect(S_DefaultReviewType);
    string jl = x['joinLink'];
    auto slogin = jl.Split("=")[1];
    title.GetServerInfo(slogin);
    yield();
    while (title.GetServerInfo_Result is null || title.GetServerInfo_Result.ServerLogin != slogin) yield();
    for (uint i = 0; i < title.GetServerInfo_Result.PlayerNames.Length; i++) {
        serverPlayers.InsertLast(title.GetServerInfo_Result.PlayerNames[i]);
    }
    for (uint i = 0; i < title.GetServerInfo_Result.ChallengeIds.Length; i++) {
        serverUids.InsertLast(title.GetServerInfo_Result.ChallengeIds[i]);
    }
    for (uint i = 0; i < title.GetServerInfo_Result.ChallengeNames.Length; i++) {
        serverMaps.InsertLast(title.GetServerInfo_Result.ChallengeNames[i]);
    }
    g_LastReviewServerUpdate = Time::Now;
    isUpdateReviewServerRunning = false;
}


void QuickJoinReviewServer() {
    ReturnToMenu(false);
    auto x = Live::GetReviewServerConnect(S_DefaultReviewType);
    string jl = x['joinLink'];
    auto slogin = jl.Split("=")[1];
    trace('joining review server: ' + jl);
    JoinServer(slogin);
}
